package main

import (
	"context"
	"fmt"
	"log"
	"os"
	"sync"
	"time"

	"git.joco.dk/sng/fermentord/internal/controllers"
	"git.joco.dk/sng/fermentord/pkg/temperature"
	"github.com/getsentry/sentry-go"
)

var (
	f *os.File
)

func persistData(ctx context.Context, wg *sync.WaitGroup, chState <-chan controllers.ChamberState, chTemp <-chan temperature.TemperatureReading) {
	var err error

	defer wg.Done()

	hub := sentry.CurrentHub().Clone()

	f, err = os.OpenFile("data.txt", os.O_WRONLY|os.O_CREATE|os.O_APPEND, 0640)
	if err != nil {
		hub.CaptureException(err)
		log.Fatal(err)
	}
	defer f.Close()

	for {
		select {
		case state, ok := <-chState:
			if !ok {
				break
			}

			err := datlog("%v STATE %v", time.Now().Unix(), controllers.ChamberStateMap[state])
			if err != nil {
				hub.CaptureException(err)
				log.Printf("Error persisting state change: %v", err)
			}

			err = f.Sync()
			if err != nil {
				hub.CaptureException(err)
				log.Printf("Error flushing data file: %v", err)
			}

		case t, ok := <-chTemp:
			if !ok {
				break
			}

			err := datlog("%v TEMP %v %v", t.Time.Unix(), t.Sensor, t.MilliDegrees)
			if err != nil {
				hub.CaptureException(err)
				log.Printf("Error persisting temperature reading: %v", err)
			}

			err = f.Sync()
			if err != nil {
				hub.CaptureException(err)
				log.Printf("Error flushing data file: %v", err)
			}

		case <-ctx.Done():
			return
		}
	}
}

func datlog(format string, data ...interface{}) error {
	_, err := fmt.Fprintf(f, format, data...)
	if err != nil {
		return err
	}

	_, err = fmt.Fprintln(f)
	if err != nil {
		return err
	}

	return nil
}
